;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Subscribe a bunch of people (with contact info dumped from exchange
;; into a file) to a given GNU mailman mailing list.
;;
;; The comments contain several approaches that were abandonded either
;; due to library issues, lack of imaginagion on the part of the
;; author or dead ends.
;;
;; I'm saving this code because it's instructive to me as an example
;; of automating web interaction with Common Lisp.
;;
;; Kyle R. Burton <kyle.burton@gmail.com>
;;
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


; (require 'asdf-install)
; (asdf-install:install :drakma)

(require :drakma)
(require :cl-ppcre)
(require :split-sequence)

(defvar *mailman-web-interface-host* "your-server-here")
(defun get-url-host (uri)
  (format nil "http://~a/~a" *mailman-web-interface-host* uri)

;; (drakma:http-request (get-url-host "/mailman/listinfo/code-reviews"))

;; doesn't parse since it's not valid xml :(
;; (require :xmls)
;; (xmls:parse (drakma:http-request (get-url-host "/mailman/listinfo/code-reviews")))

;; (asdf-install:install :cl-html-parse)
;; (require :cl-html-parse)
;; (net.html.parser:parse-html (drakma:http-request (get-url-host "/mailman/listinfo/code-reviews")))

;; (defun map-tree-leaves (fn tree &optional (acc (list)))
;;   (cond ((null tree)
;;          acc)
;;         ((not (listp tree))
;;          (funcall fn tree))
;;         (t
;;          (mapcar #'(lambda (node)
;;                       (map-tree fn node))
;;                  tree))))

;; ;;; (map-tree-leaves #'identity '(a b c (d (e (f (g h (i)))))))

;; (map-tree-leaves
;;  #'(lambda (node)
;;      (format t "node: ~a~&" node))
;;  (net.html.parser:parse-html (drakma:http-request (get-url-host "/mailman/listinfo/code-reviews"))))

;; (defun tree-visit (fn tree)
;;   (cond ((null tree)
;;          nil)
;;         ((not (listp tree))
;;          (funcall fn tree))
;;         (t
;;          (funcall fn tree)
;;          (mapcar #'(lambda (elt)
;;                      (tree-visit fn elt))
;;                  tree))))

;; (defun html-get-forms (html)
;;   (let ((forms (list)))
;;     (tree-visit
;;      #'(lambda (node)
;;          (when (and (listp node)
;;                     (listp (car node))
;;                     (equal :form (caar node)))
;;            (push node forms)))
;;      html)
;;     forms))

;; (html-get-forms (net.html.parser:parse-html (drakma:http-request (get-url-host "/mailman/listinfo/code-reviews"))))

(defun vector->list (vec)
  (loop for thing across vec
       collect thing))

(defun regex-match->groups (regex string)
  (let ((scanner (cl-ppcre:create-scanner regex :case-insensitive-mode t :multi-line-mode t :single-line-mode t)))
    (multiple-value-bind
        (string matches)
      (cl-ppcre:scan-to-strings scanner string)
    (vector->list matches))))


(let ((html (drakma:http-request (get-url-host "/mailman/listinfo/code-reviews"))))
  (regex-match->groups "(<form.+?name=\"pw\".+?</form>)" html))


;; <FORM Method=POST ACTION=\"http://mailman.some-domain.com/mailman/subscribe/code-reviews\">
;; <INPUT type=\"Text\" name=\"email\" size=\"30\" value=\"\">
;; <INPUT type=\"Text\" name=\"fullname\" size=\"30\" value=\"\">
;; <INPUT type=\"Password\" name=\"pw\" size=\"15\">
;; <INPUT type=\"Password\" name=\"pw-conf\" size=\"15\">
;; <input type=radio name=\"digest\" value=\"0\" CHECKED> No
;; <INPUT type=\"Submit\" name=\"email-button\" value=\"Subscribe\">

;; (drakma:http-request "http://www.phpsecurepages.com/test/test.php"
;;                                  :method :post
;;                                  :parameters '(("entered_login" . "test")
;;                                                ("entered_password" . "test"))
;;                                  :cookie-jar cookie-jar)

(defun subscribe-user-to-code-reviews (email full-name pass)
  (drakma:http-request (get-url-host "/mailman/subscribe/code-reviews")
                       :method :post
                       :parameters
                       `(("email"        . ,email)
                         ("fullname"     . ,full-name)
                         ("pw"           . ,pass)
                         ("pw-conf"      . ,pass)
                         ("digest"       . "0")
                         ("email-button" . "Subscribe"))))

; (subscribe-user-to-code-reviews "kburton@some-domain.com" "Kyle Burton" "asdf")

;; I went into outlook, opened the shared public contact list, did
;; CTRL-A, CTRL-C, went to excel and pasted, then saved as addrs.csv,
;; then do a bit of vim mojo to format the lines as expected below
(defun get-lines ()
  (with-open-file (in "/home/mortis/foo.txt" :direction :input)
    (loop for line = (read-line in nil nil)  then (read-line in nil nil)
       while line
       collect line)))

; (get-lines)

(defun split-line (line)
  (multiple-value-bind
        (string matches)
      (cl-ppcre:scan-to-strings "\"([^\"]+)\".*E-mail: (.+)$" line)
    (list (aref matches 0)
          (aref matches 1))))

;(split-line "\"Michel, Pete\" E-mail: pmichel@some-domain.com")

(loop for line in (get-lines)
     while line
     do
     (destructuring-bind
           (name email)
         (split-line line)
       (format t "name:~a email:~a~&" name email)
       (subscribe-user-to-code-reviews email "" "")))


(defvar *address-entries* "/home/mortis/address-book.csv")

(defun file->lines (file)
  (with-open-file (in file 
                      :direction :input
                      :external-format :latin-1)
    (loop for line = (read-line in nil nil)  then (read-line in nil nil)
       while line
       collect line)))

(defun strip-crlf (str)
  (multiple-value-bind
        (string matches)
      (cl-ppcre:scan-to-strings "^(.*)[\\r\\n]+$" str)
    (aref matches 0)))

(defun strip-quotes (str)
  (remove-if #'(lambda (ch)
                  (equal #\" ch))
             str))


(defun get-records (file)
  (split-sequence:split-sequence-if
   #'(lambda (thing)
       (equal "" thing))
   (mapcar #'strip-quotes (mapcar #'strip-crlf (file->lines file)))))

(defun record.name (rec)
  (first rec))



;; (regex-match->groups "^E-Mail: (.+)$" "E-Mail: kburton@some-domain.com")
;; (regex-match->groups "^E-Mail: (.+)$" "Foo: kburton@some-domain.com")

(defun record.email (rec)
  (let* ((pat "^E-Mail: (.+)$")
         (email (find-if #'(lambda (field)
                             (regex-match->groups pat field))
                         rec)))
    (first (regex-match->groups pat email))))


;; (record.email '("Bedoya-Valencia, Leonardo" "Bus: (XXX) YYY-ZZZZ" "Mobile: (XXX) YYY-ZZZZ"
;;                 "Bus Fax: (XXX) YYY-ZZZZ" "E-mail: lbedoya@some-domain.com"))


;(mapcar #'record.email (get-records *address-entries*))
;(mapcar #'record.name (get-records *address-entries*))



;; (require :sb-sprof)
;; (sb-sprof:with-profiling 
;;     (:max-samples 2
;;      :report :flat
;;      :show-progress t)
;;   (mapcar #'record.email (get-records *address-entries*)))


